<?php

namespace Evoluted\Deployer\VhostApi;

use function Deployer\has;
use function Deployer\get;
use function Deployer\writeln;

/**
 * Notify the Vhost API about a (possibly new) hostname
 *
 * @param string $hostname
 * @param string $applicationType E.g. "laravel", "cake"
 * @param string $phpVersion E.g. "7.1"
 * @param string $serverType Not currently used
 * @return bool Whether the request to the vhost API was successful
 */
function notifyVhostApiManager($hostname, $applicationType, $phpVersion, $serverType) {

	// If it is UAT
	if (has('vhost_api_host')) {
		$vhostApiManagerHost = get('vhost_api_host');
	} elseif (strtolower(getenv('CI_BUILD_REF_SLUG')) == 'uat' || strtolower(getenv('IS_UAT') == 'yes')) {
		$vhostApiManagerHost = 'https://vhost-api.uat.evoluted.net';
	} else {
		$vhostApiManagerHost = 'http://vhost-api.cd.evoluted.net';
	}

	if (has('application/public_path')) {
		$applicationPublicPath = get('application/public_path');
	} else {
		$applicationPublicPath = getApplicationPublicPath($applicationType);
	}

	$bodyArray = [
		'template' => 'nginx',
		'server_name' => $hostname,
		'root' => get('release_path') . DIRECTORY_SEPARATOR . $applicationPublicPath,
		'server_type' => 'php-fpm',
		'php_version' => $phpVersion
	];

	// Merge in any custom params added by the applications deployer config.
	if (has('vhost_parameters')) {
		$bodyArray = array_merge($bodyArray, get('vhost_parameters'));
	}

	$curl = curl_init();

	writeln('Sending deploy request to ' . $hostname);

	curl_setopt_array($curl, array(
		CURLOPT_URL => $vhostApiManagerHost . "/api/vhost/create",
		CURLOPT_RETURNTRANSFER => true,
		CURLOPT_ENCODING => "",
		CURLOPT_MAXREDIRS => 10,
		CURLOPT_TIMEOUT => 30,
		CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
		CURLOPT_CUSTOMREQUEST => "POST",
		CURLOPT_POSTFIELDS => json_encode($bodyArray),
		CURLOPT_HTTPHEADER => array(
			"Accept: application/json",
			"Authorization: Bearer " . getVhostManagerApiAccessToken(),
			"Content-Type: application/json",
		),
	));

	$response = curl_exec($curl);
	$err = curl_error($curl);

	curl_close($curl);

	if ($err) {
		echo "cURL Error #:" . $err;
	} else {
		var_dump($response);
		$jsonResponse = json_decode($response, true);

		if (!empty($jsonResponse['success']) && $jsonResponse['success'] == true) {
			return true;
		} else {
			return false;
		}
	}
}

/**
 * Returns the API manager access token, uses one set in deploy.php if it is with the key vhostmanagerapi/access_token
 * otherwise uses default one
 * @return string
 */
function getVhostManagerApiAccessToken() {
	if (has('vhostmanagerapi/access_token')) {
		return get('vhostmanagerapi/access_token');
	}
	return 'eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsImp0aSI6IjhmY2FhN2IyYWE4NmM0MGQwOTNkNGQyNTQ2MDYzZDY1YmIwZDY4YTkyNzRjYTQ1NmFhNDJiY2NiZGIwYzlkY2IyZDY5ODc2NmRhNGNlZDM4In0.eyJhdWQiOiIxIiwianRpIjoiOGZjYWE3YjJhYTg2YzQwZDA5M2Q0ZDI1NDYwNjNkNjViYjBkNjhhOTI3NGNhNDU2YWE0MmJjY2JkYjBjOWRjYjJkNjk4NzY2ZGE0Y2VkMzgiLCJpYXQiOjE2MjU2NjkzNDEsIm5iZiI6MTYyNTY2OTM0MSwiZXhwIjoxNjU3MjA1MzQxLCJzdWIiOiI4Iiwic2NvcGVzIjpbXX0.H61NqCkISA_wsKFlJDm7DNX5EJd1F1pg6w42Whp8XCU0C9luW2w4kSBhddKXDYa39ycACxvWCMigsqQ_V_5Kn3dKuxz3UnLQOML8aTeEKrwHrVZw8DGvbRuzADyCKU9f7k0tgeYUl-QA28-6RpfFpNoRztp16AXtpHmP2sY9ymp00NhxLkaJ-hVDKB7yT0rATBWziMHcqGajjzcklZcKjk2msOt9kQH25iDRYsoqWUbtoVe88wFonVH8vYC5z_X8h-RpH_1ar-icxbPQxQQwU2jAEwYgl5A5DmjNVHEhu6GlCK1GeMNFvUpYscePEvHU2lkMBBxISWKhlfonlQldgUbfVNuWr-UMOV91okK4J9eqiy50J8PtNI-f5TLoy2MkcgWX8KGLBSDyXUly5_5q3e4gZQyMGCIlMQbRtPqwLZpgCF45Hp6r1E7_qowDqV2eLmnkIhY55Mq559d4tf1zqvnXG4LO613OC-aqBNOW60bpyTFg7l2E8RXs1rzG6Xhb19VfK_aGt-Dgj2PsPUZNi9ovK4zVHtW0wWzQeaHMr67LZDFrt0F6kGTbM8g-GyMzJpcKXdeWtqQlVxXWhna8OOTXhmuIGlB8ArFTnVqvuMSM5jjWeq4nsV-r-V3MsPiAKHM9DeV3-hMTyx78x33eRs8hLiMcG6z_sIaFLxRTyJE';
}

/**
 * Gets the public directory path for a given application type
 *
 * @param $applicationType
 * @return string The correct relative path to the application public directory e.g. for Cakephp2 app/webroot, Laravel
 *         public
 */
function getApplicationPublicPath($applicationType) {
	switch ($applicationType) {
		case 'cake':
		case 'cakephp':
		case 'cakephp2':
			return 'app/webroot';
		case 'cakephp3':
			return 'webroot';
		case 'craftcms':
			return 'web';
		case 'laravel':
		default:
			return 'public';
	}
}
